package de.openknowledge.javafx.feedreader;

import de.openknowledge.javafx.feedreader.FeedReader;
import de.openknowledge.javafx.feedreader.FeedNews; 

import java.lang.Exception;
import java.lang.System; 

import java.net.URL;

public class FeedSubscription extends Selectable, Displayable {
  
   /** unique news feed name */
   attribute feedName : String; 
     
   /** unique news feed URL */
   attribute feedURL : String;   

   /** number of news feed entries */
   attribute numFeedNews: Number; 

   /**  operation to subscribe to a new news feed */
   operation subscribe(); 

   /** peration to refresh the current news feed*/
   operation refresh();

   /** operation to unsubscribe the current news feed */
   operation unsubscribe();
}


/** current number of news feed entries calculated via 
     lazy binding */
attribute FeedSubscription.numFeedNews = bind lazy sizeof elementList;

/** trigger, called when deleting an element from the attribute 
      elementList */
trigger on delete newsFeed from FeedSubscription.elementList { 
    /* not implemented yet */ 
} 

/** trigger, called when adding an element to the attribute 
      elementList */
trigger on insert newsFeed into FeedSubscription.elementList { 
   /* not implemented yet */
} 

/** trigger, called when changing an element to the attribute 
      elementList */
trigger on FeedSubscription.elementList[oldValue] = newValue { 
  /* not implemented yet */
}

/** trigger, called when changing the attribute feedName */
trigger on FeedSubscription.feedName[oldValue] = newValue {
  displayText = newValue;
}

/** trigger, called when changing the attribute feedURL */
trigger on FeedSubscription.feedURL[oldValue] = newValue {
   var reader:FeedReader = 
           new FeedReader(new URL(newValue)); 
   try {
     reader.read();
     for (feed in reader.entries) {

         insert new FeedNews{
               author:  "{feed.author}",
               title:   "{feed.title}",
               link:    "{feed.link}",
               uri:     "{feed.uri}",
               displayDetails: ["Autor: {feed.author}", 
                                "Titel: {feed.title} ",
                                "Link:  {feed.link}", 
                                "Uri:   {feed.uri}",
                                "",
                                "{feed.title} ..."] 
         } into elementList;         
     }

     selectedIndex = 0; 
     maxIndex = sizeof elementList; 

   } catch (IOException) {
      System.out.println("IOException occured");
   } catch (IllegalArgumentException) {
      System.out.println("IllegalArgumentException occured");
   } catch (FeedException) {
      System.out.println("FeedException occured");
   }
}

operation refresh() { /* not implemented yet */ }
operation subscribe(){ /* not implemented yet */ }
operation unsubscribe() { /* not implemented yet */ }


