package de.openknowledge.javafx.widgets; 


import java.lang.*;
import javafx.ui.*;
import javafx.ui.canvas.*;
import javafx.ui.filter.*;

function lighter(c: Color, k: Number) =
Color
{
  red: (1 - (1 - c.red) * k)
  green: (1 - (1 - c.green) * k)
  blue: (1 - (1 - c.blue) * k)
  opacity: c.opacity
};

function dim(c: Color, k: Number) =
Color
{
  red: c.red * k
  green: c.green * k
  blue: c.blue * k
  opacity: c.opacity
};

public class SizeableText
extends Text
{
  operation getBounds();
}

/**
Allows access to a protected variable in the parent class.
*/
operation SizeableText.getBounds()
{
  return this.bounds;
}

class CoolButton
extends CompositeNode
{
  public attribute text: String;
  public attribute fontFace: Font;
  public attribute fontColor: Color;
  
  public attribute color: Color;
  private attribute darkerColor: Color;
  private attribute lighterColor: Color;
  
  public attribute roundness: Number;
  public attribute inset: Number;
  public attribute x: Number;
  public attribute y: Number;
  public attribute w: Number;
  public attribute h: Number;
  
  private attribute _text: SizeableText;
  
  public attribute haloColor: Color;
  private attribute haloOffset: Number;
  private attribute haloOpacity: Number;
  private attribute storedHaloOpacity: Number;
  private attribute maxHaloOpacity: Number;
  
  public attribute onClickHaloColor: Color;
  private attribute onClickHaloOpacity: Number;
  
  public operation frame(x:Number, y: Number, w:Number, h:Number);
  public operation centerText();
  public operation getTextCenterY() : Number;
  public operation getTextCenterX() : Number;
  
  public operation onMouseEntered(e:CanvasMouseEvent);
  public operation onMouseExited(e:CanvasMouseEvent);
  public operation onMousePressed(e:CanvasMouseEvent);
  public operation onMouseReleased(e:CanvasMouseEvent);
  public operation onMouseClicked(e:CanvasMouseEvent);
  
  private operation doMouseEntered(e:CanvasMouseEvent);
  private operation doMouseExited(e:CanvasMouseEvent);
  private operation doMousePressed(e:CanvasMouseEvent);
  private operation doMouseReleased(e:CanvasMouseEvent);
  private operation doMouseClicked(e:CanvasMouseEvent);
}

attribute CoolButton.text = "";
attribute CoolButton.inset = 1;
attribute CoolButton.roundness = 1;
attribute CoolButton.w = 200;
attribute CoolButton.h = 40;
attribute CoolButton.fontFace = Font { face: VERDANA, size: 10 };
attribute CoolButton.fontColor = white;
attribute CoolButton.color = black;
attribute CoolButton.darkerColor = dim( color, 0.5  );
attribute CoolButton.lighterColor = lighter( color, 0.8  );
attribute CoolButton.haloColor = new Color( 1, 1, 1, 0.25);
attribute CoolButton.haloOffset = 2;
attribute CoolButton.haloOpacity = 0;
attribute CoolButton.storedHaloOpacity = 0;
attribute CoolButton.onClickHaloColor = new Color( 1, 1, 1, 0.25);
attribute CoolButton.onClickHaloOpacity = 0;

attribute CoolButton._text = SizeableText
{
  x: getTextCenterX()
  y: getTextCenterY()
  content: bind text
  font: bind fontFace
  fill: bind fontColor
};

trigger on CoolButton.color = newValue
{
  darkerColor = dim( color, 0.5  );
  lighterColor = lighter( color, 0.8  );
}

trigger on CoolButton.text = newValue
{
  centerText();
}

trigger on CoolButton._text = newValue
{
  centerText();
}

trigger on CoolButton.fontFace = newValue
{
  centerText();
}

trigger on CoolButton.w = newValue
{
  centerText();
}

trigger on CoolButton.h = newValue
{
  centerText();
}

operation CoolButton.frame(x:Number, y: Number, w:Number, h:Number)
{
  this.x = x;
  this.y = y;
  this.w = w;
  this.h = h;
}

operation CoolButton.centerText()
{
  _text.y = getTextCenterY();
  _text.x = getTextCenterX();
}

operation CoolButton.getTextCenterY() : Number
{
  var boxHeight = this.h;
  var textY = ( ( boxHeight - _text.getBounds().height ) / 2 );
  textY += this.y;
  return textY;
}

operation CoolButton.getTextCenterX() : Number
{
  var boxWidth = this.w;
  var textX = ( ( boxWidth - _text.getBounds().width ) / 2 );
  textX += this.x;
  return textX;
}

operation CoolButton.doMouseEntered(e:CanvasMouseEvent)
{
  if( onClickHaloOpacity == 0 )
  {
    haloOpacity = 1;
  }
  else
  {
    storedHaloOpacity = 1;
  }
  
  //forward this to any subclasses
  onMouseEntered(e);
}
operation CoolButton.doMouseExited(e:CanvasMouseEvent)
{
  haloOpacity = 0;
  storedHaloOpacity = 0;
  
  //forward this to any subclasses
  onMouseExited(e);
}

operation CoolButton.doMousePressed(e:CanvasMouseEvent)
{
  storedHaloOpacity = haloOpacity;
  haloOpacity = 0;
  onClickHaloOpacity = 1;
  
  //forward this to any subclasses
  onMousePressed(e);
}

operation CoolButton.doMouseReleased(e:CanvasMouseEvent)
{
  onClickHaloOpacity = 0;
  haloOpacity = storedHaloOpacity;
  
  //forward this to any subclasses
  onMouseReleased(e);
}

operation CoolButton.doMouseClicked(e:CanvasMouseEvent)
{
  //forward this to any subclasses
  onMouseClicked(e);
}

function CoolButton.composeNode() =
Group
{
  content:
  [
    Group
    {
      filter: [GaussianBlur {radius: 2}]
      content:
      [
        Rect
        {
          x: bind x
          y: bind y
          width: bind w
          height: bind h
          arcHeight: bind roundness * h
          arcWidth: bind roundness * h
          fill: bind LinearGradient
          {
            x1: 0, y1: 0, x2: 0, y2: 1
            stops:
            [
            Stop { offset: 0.0 color: bind darkerColor },
            Stop { offset: 1.0 color: bind color }
            ]
            spreadMethod: PAD
          }
          
          onMouseEntered: operation(e:CanvasMouseEvent)
          {
            doMouseEntered(e);
          }
          
          onMouseExited: operation(e:CanvasMouseEvent)
          {
            doMouseExited(e);
          }
          
          onMousePressed: operation(e:CanvasMouseEvent)
          {
            doMousePressed(e);
          }
          
          onMouseReleased: operation(e:CanvasMouseEvent)
          {
            doMouseReleased(e);
          }
          
          onMouseClicked: operation(e:CanvasMouseEvent)
          {
            doMouseClicked(e);
          }
        },
        
        Clip
        {
          shape: Rect
          {
            x: bind x+inset
            y: bind y+inset
            width: bind w-2*inset
            height: bind (h-2*inset) /2
            arcHeight: bind roundness * h /3
            arcWidth: bind roundness * h /3
          }
          content: Rect
          {
            x: bind x+inset
            y: bind y+inset
            width: bind w-2*inset
            height: bind h-2*inset
            arcHeight: bind roundness * h
            arcWidth: bind roundness * h
            opacity: 0.8
            fill: bind LinearGradient
            {
              x1: 0, y1: 0, x2: 0, y2: 0.5
              stops:
              [
              Stop { offset: 0.0 color: white },
              Stop { offset: 1.0 color: bind lighterColor }
              ]
              spreadMethod: PAD
            }
          }
        },
        
        Rect
        {
          x: bind x + haloOffset
          y: bind y + haloOffset
          width: bind w
          height: bind h
          arcHeight: bind roundness * h
          arcWidth: bind roundness * h
          fill: bind haloColor
          opacity: bind haloOpacity
        },
        
        Rect
        {
          x: bind x + haloOffset
          y: bind y + haloOffset
          width: bind w
          height: bind h
          arcHeight: bind roundness * h
          arcWidth: bind roundness * h
          stroke: bind onClickHaloColor
          strokeWidth: 2
          opacity: bind onClickHaloOpacity
        },
      ]
    },
    
    _text
  ]
};

/*
This is an anoying hack.  I can't figure out how to get the mouse events
I need to know that this button has been hovered over,pressed, etc 
without eating the events that let me know it's been clicked.  For
some reason, this subclass works fine.
*/
public class ClickedButton extends CoolButton { }

attribute ClickedButton.text = "";
attribute ClickedButton.x = 25;
attribute ClickedButton.y = 25;
attribute ClickedButton.fontFace = Font { face: VERDANA, size: 10 };


/**
Anoying hack #2: text can only be centered after the element is visible; adding
this to the on new trigger, etc, does nothing.
*/