package de.openknowledge.javafx.widgets; 

import javafx.ui.*;
import javafx.ui.canvas.*;
import javafx.ui.filter.*;
import java.lang.*;

import de.openknowledge.javafx.feedreader.Selectable; 

import de.openknowledge.javafx.widgets.CoolButton; 
import de.openknowledge.javafx.widgets.GoToFirstNavigationButton; 
import de.openknowledge.javafx.widgets.GoToPreviousNavigationButton; 
import de.openknowledge.javafx.widgets.GoToNextNavigationButton; 
import de.openknowledge.javafx.widgets.GoToLastNavigationButton; 
            
operation brighter( c: Color ) {
    var newColor = c.getColor().brighter();
    return rgba:Color(newColor.getRed(), newColor.getGreen(), newColor.getBlue(), newColor.getAlpha());
}

operation darker( c: Color ) {
    var newColor = c.getColor().darker();
    return rgba:Color(newColor.getRed(), newColor.getGreen(), newColor.getBlue(), newColor.getAlpha());
}

public class SelectionBox extends CompositeNode
{

   attribute arcSize: Integer;
   attribute titleBarHeight: Integer;
   attribute buttonBarHeight: Integer;

    attribute displayType: String; 

    attribute textColor: Color; 
    attribute color: Color;
    attribute roundness: Number;
    attribute title: String;

    attribute inset: Number;

    attribute selectableList: Selectable;
    attribute selectedIndex: Integer; 

    attribute goFirstButton: GoToFirstNavigationButton; 
    attribute goLastButton: GoToLastNavigationButton; 
    attribute goBackButton: GoToPreviousNavigationButton;
    attribute goForwardButton: GoToNextNavigationButton; 

    attribute x: Number;
    attribute y: Number;
    attribute w: Number;
    attribute h: Number;

    operation frame(x:Number, y: Number, w:Number, h:Number);

}

attribute SelectionBox.inset = 0;
attribute SelectionBox.roundness = 1.0;
attribute SelectionBox.title = "Unknown";

attribute  SelectionBox.arcSize = 30;
attribute SelectionBox.titleBarHeight = 20;
attribute SelectionBox.buttonBarHeight = 25;

attribute SelectionBox.selectedIndex = bind selectableList.selectedIndex;

attribute SelectionBox.goFirstButton = new GoToFirstNavigationButton {
       w:     40,
       h:     15,
       x:     bind x + w/2 - 120,
       y:     bind y + h - buttonBarHeight + 5,
      selectableList: bind selectableList
}; 


attribute SelectionBox.goBackButton = new GoToPreviousNavigationButton {
      w:     40,
      h:     15,
      x:     bind x + w/2 - 120 + 60,
      y:     bind y + h - buttonBarHeight + 5,
      selectableList: bind selectableList
};

attribute SelectionBox.goForwardButton = new GoToNextNavigationButton {
      w:     40,
      h:     15,
      x:     bind x + w/2 - 120 + 60 + 60,
      y:     bind y + h - buttonBarHeight + 5,
      selectableList: bind selectableList
};

attribute SelectionBox.goLastButton = new  GoToLastNavigationButton {
      w:     40,
      h:     15,
      x:     bind x + w/2 - 120 + 60 + 60 + 60,
      y:     bind y + h - buttonBarHeight + 5,
      selectableList: bind selectableList
};

/** Trigger welche den displayName der Mutterklasse Selectable 
    auf den Wert des feedName setzt */
trigger on SelectionBox.selectedIndex[oldValue] = newValue {
   System.out.println("Navigation: Set selectedIndex from {oldValue} to {newValue}");

   foreach (element in selectableList.elementList 
        where indexof element <10) ( 
            if (indexof element < 9) then 
               Text {
                  x: bind x + 10
                  y: bind y + 10 + titleBarHeight + (indexof element * 20)
                  content: bind element.displayText
                  fill: if (indexof element == selectableList.selectedIndex) 
                          then rgb:Color(255,255,255)
                          else rgb:Color(150,150,150)
               }
           else 
              Text {
                  x: bind x + 10 + 140
                  y: bind y + 10 + titleBarHeight + 180
                  content: "... weitere Meldungen ..."
                  fill: rgb:Color(150,150, 150)
                  // font: Font {face: VERDANA, style: [BOLD], size: 10}
               }
}

operation SelectionBox.frame(x:Number, y: Number, w:Number, h:Number)
{
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
}

function SelectionBox.composeNode() =
Group
{
      content:
    [ Rect {
        x: bind x
        y: bind y
        width: bind w
        height: bind h
        arcHeight: bind roundness * arcSize
        arcWidth: bind roundness * arcSize
        fill: color
        stroke: black
        onMouseEntered: operation( e ) {
            color = brighter( color );
        }
        onMouseExited: operation( e ) {
            color = darker( color );
        }
    } ,

  // footer section
  Add {
        shape1: Rect {
            x: bind x
            y: bind y + h - buttonBarHeight
            width: bind w
            height: buttonBarHeight
            arcHeight: bind roundness * arcSize
            arcWidth: bind roundness * arcSize
        }
        shape2: Rect {
            x: bind x
            y: bind y + h - buttonBarHeight
            width: bind w
            height: buttonBarHeight/2
        }
        fill: bind LinearGradient
        {
            x1: 0, y1: 1, x2: 0, y2: 0
            stops:
            [
            Stop {offset: 0.0 color: bind darker( darker( darker
                (color) ) )},
            Stop {offset: 1.0 color: bind brighter( brighter( brighter
                (color) ) )}
            ]
        }
        stroke: bind darker( color )
        strokeWidth: 1
    },
  
    // header section
    Add {
        shape1: Rect {
            x: bind x
            y: bind y
            width: bind w
            height: titleBarHeight
            arcHeight: bind roundness * arcSize
            arcWidth: bind roundness * arcSize
        }
        shape2: Rect {
            x: bind x
            y: bind y + titleBarHeight/2
            width: bind w
            height: titleBarHeight/2
        }
        fill: bind LinearGradient
        {
            x1: 0, y1: 0, x2: 0, y2: 1
            stops:
            [
            Stop {offset: 0.0 color: bind darker( darker( darker
                (color) ) )},
            Stop {offset: 1.0 color: bind brighter( brighter( brighter
                (color) ) )}
            ]
        }
        stroke: bind darker( color )
        strokeWidth: 1
    },


    Clip
    {
        shape: Rect
        {
            x: bind x+inset
            y: bind y+inset
            width: bind w-2*inset
            height: (titleBarHeight - 2*inset) / 2
            arcHeight: bind roundness * arcSize
            arcWidth: bind roundness * arcSize
        }
        content: Rect
        {
            x: bind x+inset
            y: bind y+inset
            width: bind w-2*inset
            height: titleBarHeight - 2*inset
            arcHeight: bind roundness * arcSize
            arcWidth: bind roundness * arcSize
            opacity: 0.8
            fill: bind LinearGradient
            {
                x1: 0, y1: 0, x2: 0, y2: 0.5
                stops:
                [
                Stop {offset: 0.0 color: white},
                Stop {offset: 1.0 color: bind brighter( color )}
                ]
            }
        }
    },

    goFirstButton, 
    goBackButton, 
    goForwardButton, 
    goLastButton,
 
  // header Text 
    Text { content: bind title, x: bind x + 10, y: bind y + 5, fill: white },   
   
   if (displayType == "list") then (
      foreach (element in selectableList.elementList 
        where indexof element <10) ( 
            if (indexof element < 9) then 
               Text {
                  x: bind x + 10
                  y: bind y + 10 + titleBarHeight + (indexof element * 20)
                  content: bind element.displayText
                  fill: if (indexof element == selectableList.selectedIndex) 
                          then rgb:Color(255,255,255)
                          else rgb:Color(150,150,150)
               }
           else 
              Text {
                  x: bind x + 10 + 140
                  y: bind y + 10 + titleBarHeight + 180
                  content: "... weitere Meldungen ..."
                  fill: rgb:Color(150,150, 150)
                  // font: Font {face: VERDANA, style: [BOLD], size: 10}
               }

      )
   ) else ( // "detail"

      foreach (elementDetail in selectableList.elementList[selectableList.selectedIndex].displayDetails)
         Text {
                  x: bind x + 10
                  y: bind y + 10 + titleBarHeight + (indexof elementDetail * 20)
                  content: bind elementDetail
                  fill: if (indexof elementDetail == 5) // content
                          then rgb:Color(255,255,255)
                          else rgb:Color(150,150,150)
                  // font: Font {face: VERDANA, style: [BOLD], size: 10}
               }
    )
]
};
